variable "output_directory" {
  type    = string
  default = "output"
}

variable "checksum_directory" {
  type    = string
  default = "checksums"
}

variable "vm_name" {
  type    = string
  default = "cloudimg"
}

variable "iso_url" {
  type    = string
  default = "output/ubuntu-20-04/ubuntu-20-04.qcow2"
}

variable "iso_checksum" {
  type    = string
  default = "none"
}

variable "cpus" {
  type    = number
  default = 1
}

variable "memsize" {
  type    = number
  default = 1024
}

variable "qemu_accelerator" {
  type    = string
  default = "kvm"
}

variable "disk_size" {
  type    = string
  default = "8G"
}

variable "disk_detect_zeroes" {
  type    = string
  default = "on"
}

variable "disk_compression" {
  type    = bool
  default = true
}

variable "disk_format" {
  type    = string
  default = "qcow2"
}

variable "username" {
  type    = string
  default = "student"
}

variable "password" {
  type    = string
  default = "student"
}

variable "headless" {
  type    = bool
  default = false
}

variable "img_name" {
  type    = string
  default = "img"
}

variable "include_disks" {
  type    = set(string)
}

source "qemu" "cloudimg" {
  iso_url            = var.iso_url
  iso_checksum       = var.iso_checksum
  disk_image         = true
  use_backing_file   = true
  output_directory   = "${var.output_directory}/${var.vm_name}"
  cpus               = var.cpus
  memory             = var.memsize
  disk_size          = var.disk_size
  disk_interface     = "virtio"
  disk_compression   = var.disk_compression
  disk_detect_zeroes = var.disk_detect_zeroes
  format             = var.disk_format
  accelerator        = var.qemu_accelerator
  ssh_username       = var.username
  ssh_password       = var.password
  ssh_timeout        = "30m"
  shutdown_command   = "rm -rf ~/.ansible && echo '${var.password}' | sudo -S poweroff"
  vm_name           = "${var.img_name}"
  net_device         = "virtio-net"
  headless           = var.headless
}

build {
  sources = ["source.qemu.cloudimg"]

  provisioner "ansible" {
    playbook_file    = "scripts/ansible/${var.vm_name}.yml"
    user             = var.username
    use_proxy        = false
    extra_arguments  = [
      "--extra-vars", "ansible_password='${var.password}' ansible_become_pass='${var.password}'",
      "--extra-vars", "include_disks=${format("%v", var.include_disks)}",
      "--extra-vars", "hostname='${var.vm_name}'",
    ]
  }

  post-processor "shell-local" {
    inline = ["rm -f ${var.checksum_directory}/${var.vm_name}.*"]
  }

  post-processor "shell-local" {
      inline = ["qemu-img snapshot -c new '${var.output_directory}/${var.vm_name}/${var.img_name}'"]
  }

  post-processor "checksum" {
    checksum_types = ["sha256", "sha512"]
    output = "${var.checksum_directory}/${var.vm_name}.{{.ChecksumType}}"
  }

  post-processor "shell-local" {
    inline = ["sed -Ei 's/${var.img_name}([^ ]*)$/${var.vm_name}.${var.disk_format}\\1/g' ${var.checksum_directory}/${var.vm_name}.*"]
  }
}
