#!/bin/bash

set -u

crtscript=$(readlink -e "$0")
basedir="${crtscript%/*}"
network="labvms"
labdirs=($(find "${basedir}" -type d -name "lab-*"))
ipprefix=192.168.100
usernet_xml="${basedir}/usernet.xml"
utils_script="${basedir}/utils.sh"
bashrc="/home/student/.bashrc"

sudo apt update
sudo apt install -y sshpass

sudo virsh net-define "/usr/share/libvirt/networks/${network}.xml" 2>/dev/null
sudo virsh net-autostart "${network}" 2>/dev/null
sudo virsh net-start "${network}" 2>/dev/null

cat <<EOF >"${usernet_xml}"
<network>
  <name>usernet</name>
  <bridge name='usernet'/>
  <forward/>
  <ip address="10.10.0.0" netmask="255.255.0.0"/>
  <ip family="ipv6" address="2200::1" prefix="60"/>
</network>
EOF

sudo virsh net-define "${usernet_xml}" 2>/dev/null
sudo virsh net-autostart "usernet" 2>/dev/null
sudo virsh net-start "usernet" 2>/dev/null

sudo ip a d 10.10.0.0/16 dev usernet
sudo ip a d 2200::1/60 dev usernet

for labdir in "${labdirs[@]}"; do
    dir=$(readlink -e "${labdir}")
    source "${dir}/config"
    ip="${ipprefix}.${hostip}"

    sudo virsh net-update "${network}" add ip-dhcp-host \
        "<host mac='${macaddr}' name='${vmname}' ip='${ip}'/>" 2>/dev/null

    disks=($(find "${dir}" -maxdepth 1 -type f -name '*.qcow2' | sort -V))
    disk_args=$(printf -- "--disk path='%s',format=qcow2 " "${disks[@]}")

    sudo virt-install --name "${vmname}" --hvm --os-variant generic --ram "${memory}" \
        --vcpus "cores=${cores}" --network network="${network}",mac="${macaddr}"      \
        --network network="usernet"                                                   \
        --graphics vnc,listen=127.0.0.1 --noautoconsole --import --virt-type kvm ${disk_args}
done

grep -q -- "${utils_script}" "${bashrc}" || echo "source '${utils_script}'" >> "${bashrc}"
