#!/bin/bash

VM_NAMES=("lab-routing-1" "lab-routing-2" "lab-routing-3")

declare -A VM_IP_SHORTHAND
VM_IP_SHORTHAND["red"]="192.168.100.111"
VM_IP_SHORTHAND["green"]="192.168.100.112"
VM_IP_SHORTHAND["blue"]="192.168.100.113"

function go() {
    case "$1" in
        red|green|blue)
            sshpass -p student ssh -o "StrictHostKeyCHecking=accept-new" student@"${VM_IP_SHORTHAND[$1]}"
            ;;
        *)
            echo "Wrong argument. Usage: go red|green|blue"
            ;;
    esac
}

function clean_host () {
    sudo ip address flush usernet
    sudo sysctl -w net.ipv4.ip_forward=0
}

function reboot_vms () {
    echo "Rebooting VMs"
    for vm in "${VM_NAMES[@]}"
    do
        virsh reboot "$vm" &> /dev/null
    done
    sleep 5
    for vm in "red" "green" "blue"
    do
        wait_vm "$vm"
    done
}

function wait_vm () {
    echo -n "Waiting for $1 "
    until nc -vzw 2 "${VM_IP_SHORTHAND[$1]}" 22 &> /dev/null
    do
        echo -n "."
        sleep 2
    done
    echo " Done!"
}

function start_lab () {
    case "$1" in
        ex6)
            clean_host &> /dev/null
            reboot_vms
            prepare_6 &> /dev/null
            ;;
        ex7)
            clean_host &> /dev/null
            reboot_vms
            prepare_7 &> /dev/null
            ;;
        ex9)
            clean_host &> /dev/null
            reboot_vms
            prepare_9 &> /dev/null
            ;;
        *)
            echo "Wrong argument."
            ;;
    esac
}

function vm_exec () {
    sshpass -p student ssh -o "StrictHostKeyChecking=accept-new" student@"${VM_IP_SHORTHAND[$1]}" "${@:2}"
}

function prepare_6 () {
    sudo ip address add 10.10.7.1 dev usernet
    vm_exec red sudo ip address add 10.10.7.2/24 dev red-eth0

    vm_exec red sudo ip link set up dev red-eth0
    sudo sysctl -w net.ipv4.ip_forward=1
    vm_exec red sudo ip route add default via 10.10.7.1
}

function prepare_7 () {
    sudo ip address add 10.10.15.0 dev usernet
    vm_exec blue sudo ip address add 10.10.15.2 dev blue-eth0

    for vm in "red" "green" "blue"
    do
        vm_exec $vm sudo ip link set up dev "$vm-eth0"
    done

    sudo ip address add 10.10.10.1/24 dev usernet
    sudo ip address add 10.10.20.1/24 dev usernet

    vm_exec red sudo ip address add 10.10.10.2/24 dev red-eth0
    vm_exec green sudo ip address add 10.10.20.2/24 dev green-eth0

    vm_exec red sudo ip route add default via 10.10.10.1
    vm_exec green sudo ip route add default via 10.10.20.1

    sudo sysctl -w net.ipv4.ip_forward=1
    vm_exec blue sudo ip link set down dev blue-eth0
}

function prepare_9 () {
    sudo ip address add 10.10.10.1/24 dev usernet
    sudo sysctl -w net.ipv4.ip_forward=1
}
